/*
 * Copyright CoreMedia AG, Hamburg, 2009. All rights reserved.
 */
package com.coremedia.jira.plugins.cloneandmove.web.action.issue;

import com.atlassian.jira.config.SubTaskManager;
import com.atlassian.jira.config.properties.ApplicationProperties;
import com.atlassian.jira.issue.AttachmentManager;
import com.atlassian.jira.issue.Issue;
import com.atlassian.jira.issue.IssueFactory;
import com.atlassian.jira.issue.IssueFieldConstants;
import com.atlassian.jira.issue.fields.CustomField;
import com.atlassian.jira.issue.fields.FieldManager;
import com.atlassian.jira.issue.fields.SummarySystemField;
import com.atlassian.jira.issue.fields.layout.field.FieldLayoutStorageException;
import com.atlassian.jira.issue.link.IssueLinkManager;
import com.atlassian.jira.issue.link.IssueLinkTypeManager;
import com.atlassian.jira.security.PermissionManager;
import com.atlassian.jira.web.action.issue.CloneIssueDetails;
import com.atlassian.jira.web.action.issue.IssueCreationHelperBean;
import org.ofbiz.core.entity.GenericEntityException;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <p> This action will clone an issue and move it to another project. Unlike the standard {@link CloneIssueDetails} it
 * prevents the title to be enriched by the "CLONE" keyword. In addition you might configure fields which will not be
 * cloned. This might be useful e. g. for switches in the tasks which represent a certain state of the issue. </p>
 *
 * @author Mark Michaelis
 */
public class CloneAndMoveIssueDetails extends CloneIssueDetails {
  private final FieldManager fieldManager;
  /**
   * You may use this pattern in the description of your custom field it to prevent it from being cloned.
   * Useful for example for issue state switches or Version-Control-Job-Switches etc.
   */
  private static final Pattern NOCLONE_PATTERN = Pattern.compile(".*<!--\\s*noclone\\s*-->.*");

  /**
   * Constructor which also makes several utility and management classes available.
   *
   * @param applicationProperties   This can be used to lookup JIRA application properties.
   * @param permissionManager       Manager responsible for JIRA Permissions.
   * @param issueLinkManager        The implementations of this class are used to manage issue link types and issue
   *                                links.
   * @param issueLinkTypeManager    Manages IssueLinkTypes.
   * @param subTaskManager          Manages SubTasks - issues that are "part of" other issues.
   * @param attachmentManager       Manages all attachment related tasks in JIRA, which involves retrieving an
   *                                attachment, creating an attachment and deleting an attachment.
   * @param fieldManager            Manager to access fields; also custom fields.
   * @param issueCreationHelperBean helper for issue creation; also supports validation
   * @param issueFactory            The IssueFactory is used for creating Issues in JIRA, as well as converting
   *                                GenericValue issue objects to proper Issue objects.
   */
  public CloneAndMoveIssueDetails(final ApplicationProperties applicationProperties,
                                  final PermissionManager permissionManager,
                                  final IssueLinkManager issueLinkManager,
                                  final IssueLinkTypeManager issueLinkTypeManager,
                                  final SubTaskManager subTaskManager, final AttachmentManager attachmentManager,
                                  final FieldManager fieldManager,
                                  final IssueCreationHelperBean issueCreationHelperBean,
                                  final IssueFactory issueFactory) {
    super(applicationProperties, permissionManager, issueLinkManager, issueLinkTypeManager, subTaskManager,
            attachmentManager, fieldManager, issueCreationHelperBean, issueFactory);
    this.fieldManager = fieldManager;
  }

  /**
   * Set cloned issue field "summary" to same values as in original issue. Unlike {@link
   * CloneIssueDetails#setIssueDetails(com.atlassian.jira.issue.Issue)} it does not prepend the "CLONE" prefix to the
   * issue.
   *
   * @param issue issue to copy the field values to
   *
   * @throws GenericEntityException in case of an error
   */
  @Override
  public void setIssueDetails(final Issue issue) throws GenericEntityException {
    final SummarySystemField summaryField =
            (SummarySystemField) fieldManager.getOrderableField(IssueFieldConstants.SUMMARY);
    summaryField.populateFromIssue(getFieldValuesHolder(), getOriginalIssue());
  }

  /**
   * Custom Fields may prevent cloning by adding the string <code>&lt;!-- noclone -->&gt;</code> to their description
   * field.
   *
   * @throws FieldLayoutStorageException in case the new field values cannot be stored
   */
  @Override
  protected void setFields() throws FieldLayoutStorageException {
    super.setFields();
    final List /* <CustomField> */ customFields = getCustomFields(getOriginalIssue());

    for (final Object customField1 : customFields) {
      final CustomField customField = (CustomField) customField1;
      final String description = customField.getDescription();
      final Matcher matcher = NOCLONE_PATTERN.matcher(description);
      if (matcher.matches()) {
        getIssueObject().setCustomFieldValue(customField, null);
      }
    }
  }

  /**
   * Determine what to do next after the issue has been cloned. The standard clone-and-move task will show the newly
   * created task. Clone And Move instead will continue with moving the task.
   *
   * @return Redirect URL to new page
   *
   * @throws Exception in case of an error
   */
  @Override
  protected String doPostCreationTasks() throws Exception {
    if (getCloneParent() != null) {
      // If an issue has been cloned - redirect to move issue page
      return getRedirect("MoveIssue!default.jspa?id=" + getCloneParent().getString("id"));
    }
    else {
      // If a subtask has been cloned or an issue with no sub tasks - redirect to move subtask issue page
      return getRedirect("MoveSubTaskChooseOperation!default.jspa?id=" + getIssue().getString("id"));
    }
  }

}
